import json
import logging

from celery import bootsteps, Task
import redis
from lxml import etree

from inventory_provider.tasks.app import app
from inventory_provider import config
from inventory_provider import constants
from inventory_provider import environment
from inventory_provider import snmp
from inventory_provider import juniper

environment.setup_logging()


class InventoryTask(Task):

    config = None
    logger = None

    def __init__(self):
        pass

    @staticmethod
    def save_key(hostname, key, value):
        assert isinstance(value, str), \
            "sanity failure: expected string data as value"
        r = redis.StrictRedis(
            host=InventoryTask.config["redis"]["hostname"],
            port=InventoryTask.config["redis"]["port"])
        r.hset(
            name=hostname,
            key=key,
            value=value)
        InventoryTask.logger.debug(
            "saved %s, key %s" % (hostname, key))
        return "OK"

    @staticmethod
    def save_key_json(hostname, key, data_obj):
        InventoryTask.save_key(
            hostname,
            key,
            json.dumps(data_obj))

    @staticmethod
    def save_key_etree(hostname, key, xml_doc):
        InventoryTask.save_key(
            hostname,
            key,
            etree.tostring(xml_doc, encoding='unicode'))


class WorkerArgs(bootsteps.Step):
    def __init__(self, worker, config_filename, **options):
        with open(config_filename) as f:
            InventoryTask.config = config.load(f)
        InventoryTask.logger = logging.getLogger(constants.TASK_LOGGER_NAME)


def worker_args(parser):
    parser.add_argument(
        "--config_filename",
        dest="config_filename",
        action='store',
        type=str,
        help="Configuration filename")


app.user_options['worker'].add(worker_args)
app.steps['worker'].add(WorkerArgs)


@app.task(bind=InventoryTask)
def snmp_refresh_interfaces(self, hostname, community):
    logger = logging.getLogger(constants.TASK_LOGGER_NAME)
    logger.debug('STARTING: snmp_refresh_interfaces(%r, %r)'
                 % (hostname, community))

    InventoryTask.save_key_json(
        hostname,
        "snmp-interfaces",
        list(snmp.get_router_interfaces(
            hostname,
            community,
            InventoryTask.config)))

    logger.debug('FINISHED: snmp_refresh_interfaces(%r, %r)'
                 % (hostname, community))


@app.task(bind=InventoryTask)
def netconf_refresh_config(self, hostname):
    logger = logging.getLogger(constants.TASK_LOGGER_NAME)
    logger.debug('STARTING: netconf_refresh_config(%r)' % hostname)

    InventoryTask.save_key_etree(
        hostname,
        "netconf",
        juniper.load_config(hostname, InventoryTask.config["ssh"]))

    logger.debug('FINISHED: netconf_refresh_config(%r)' % hostname)
